unit interpreter;

interface

uses Classes, SysUtils, InterpreterProperty, DsgnIntf;

const
   DefaultTokenStartString: char= '<';
   DefaultTokenEndString  : char= '>';

type
   TInOutStrings= class(TStringlist)
   end;

   TInterpreterDelimiter= class(TPersistent)
      private
        fStartString,
        fEndString: char;
      public
        constructor create;
      published
        property StartString: char read fStartString write fStartString;
        property EndString: char read fEndString write fEndString;
   end;

   TInterpreterEvent= procedure(Sender: TObject; Code, Params: string; var Handled: Boolean) of object;
   TInterpreterFunc = function(Params: string): string of object;

   TInterpreterToken= class(TObject)
      private
        fExecute: TInterpreterFunc;
      protected
        constructor create(AFunction: TInterpreterFunc);
      public
        property Execute: TInterpreterFunc read fExecute;
   end;

   TInterpreter=class(TComponent)
     private
       fAutomatic: Boolean;
       fToken: TStringlist;
       fInput: TInOutStrings;
       fOutput: TInOutStrings;
       fDelimiter: TInterpreterDelimiter;
       fOnRefresh: TNotifyEvent;
       fOnToken: TInterpreterEvent;
       procedure SetInput(AContent: TInOutStrings);
       procedure Execute;
     public
       constructor create(AOwner: TComponent); override;
       destructor destroy; override;
       function AddToken(Code: string; InterpreterFunc: TInterpreterFunc): Boolean;
       procedure Refresh;
       property Token: TStringlist read fToken;
     published
       property Automatic: Boolean read fAutomatic write fAutomatic;
       property Delimiter: TInterpreterDelimiter read fDelimiter write fDelimiter;
       property Input: TInOutStrings read fInput write SetInput;
       property Output: TInOutStrings read fOutput;
       property OnRefresh: TNotifyEvent read fOnRefresh write fOnRefresh;
       property OnToken: TInterpreterEvent read fOnToken write fOnToken;
   end;

procedure Register;

implementation

{$R interpreter.res}

procedure Register;
begin
   RegisterComponents('PAConline', [TInterpreter]);
   RegisterPropertyEditor(TypeInfo(TInOutStrings),
                          TInterpreter,
                          'Input',
                          TInterpreterProperty);
   RegisterPropertyEditor(TypeInfo(TInOutStrings),
                          TInterpreter,
                          'Output',
                          TInterpreterProperty);
end;

(****************************************************
 Procedure TInterpreterDelimiter.create
 Unit      interpreter
 Project   INTERPRETER2 / Spirit GmbH
 Author    Christian Aust
 Date      01.02.99 11:44:36
 ****************************************************)
constructor TInterpreterDelimiter.create;
begin
   fStartString := DefaultTokenStartString;
   fEndString   := DefaultTokenEndString;
end;

(****************************************************
 Procedure TInterpreterToken.create
 Unit      interpreter
 Project   CODER / Spirit GmbH
 Author    Christian Aust
 Date      28.01.99 17:42:12
 ****************************************************)
constructor TInterpreterToken.create(AFunction: TInterpreterFunc);
begin
   inherited create;
   fExecute:= AFunction;
end;

(****************************************************
 Procedure TInterpreter.Refresh
 Unit      interpreter
 Project   INTERPRETER2 / Spirit GmbH
 Author    Christian Aust
 Date      01.02.99 12:22:02
 ****************************************************)
procedure TInterpreter.Refresh;
begin
   Execute;
end;

(****************************************************
 Procedure TInterpreter.AddToken
 Unit      interpreter
 Project   CODER / Spirit GmbH
 Author    Christian Aust
 Date      28.01.99 20:03:11
 ****************************************************)
function TInterpreter.AddToken(Code: string; InterpreterFunc: TInterpreterFunc): Boolean;
begin
   if fToken.IndexOf(lowercase(Code))=-1 then begin
      fToken.AddObject(lowercase(Code), TInterpreterToken.Create(InterpreterFunc));
      Result:= true;
   end else Result:= false;
end;

(****************************************************
 Procedure TInterpreter.Execute
 Unit      interpreter
 Project   CODER / Spirit GmbH
 Author    Christian Aust
 Date      28.01.99 18:14:09
 ****************************************************)
procedure TInterpreter.Execute;
var
   Eingang: string;
   i, j: Integer;
   Tokenstart: Integer;
   InToken: Boolean;
   TokenText: string;
   Code, Params: string;
   TokenHandled: Boolean;
begin
   Eingang:= fInput.Text;
   i:= 0;
   Tokenstart:= -1;
   InToken:= false;
   while i<length(Eingang) do begin
      if not(Eingang[i] in [#$A, #$D]) then begin //CR/LF ignorieren
         // '<' markiert den Anfang des Tokens, << sperrt
         if not InToken and (Eingang[i]=fDelimiter.StartString) and (Eingang[i+1]<>fDelimiter.StartString) then begin
            Tokenstart:= i+1;
            InToken:= true;
         end;
         if InToken and (Eingang[i]=fDelimiter.EndString) and (Eingang[i+1]<>fDelimiter.EndString) then begin
            //Token gefunden
            TokenText:= Copy(Eingang, Tokenstart, i-TokenStart);
            //Token rausschneiden
            Delete(Eingang, TokenStart-1, i-TokenStart+2);
            //und Suchmarke entsprechend verschieben
            i:= i-length(Tokentext)-2;
            Tokentext:= Trim(Tokentext);
            if Length(Tokentext)>0 then begin
               //Tokentext ersetzen
               j:= 0;
               while j<=length(Tokentext) do begin
                  if Tokentext[j] in [#$A, #$D] then
                     Delete(Tokentext, j, 1)
                  else inc(j);
               end;
               j:= Pos(' ', Tokentext);
               if j>0 then begin //Token mit Parametern ?
                  Code:= LowerCase(Copy(Tokentext, 1, j-1));
                  Delete(Tokentext, 1, j);
                  Params:= Tokentext;
               end else begin
                  Code:= LowerCase(Tokentext);
                  Params:= '';
               end;
               TokenHandled:= false;
               if Assigned(fOnToken) then
                  fOnToken(self, Code, Params, TokenHandled);
               j:= 0;
               if (not TokenHandled) and fToken.Find(Code, j) then begin
                  TokenText:= (fToken.Objects[j] as TInterpreterToken).Execute(Params);
                  Insert(TokenText, Eingang, TokenStart-1);
                  inc(i, length(Tokentext));
               end;
            end;
            //Flag wieder zurcksetzen
            TokenStart:= -1;
            InToken:= false;
         end;
      end; //if not CR/LF
      inc(i);
   end; //while
   fOutput.Text:= Eingang;
   if Assigned(fOnRefresh) then fOnRefresh(self);
end;

(****************************************************
 Procedure TInterpreter.SetInput
 Unit      interpreter
 Project   CODER / Spirit GmbH
 Author    Christian Aust
 Date      28.01.99 17:51:25
 ****************************************************)
procedure TInterpreter.SetInput(AContent: TInOutStrings);
begin
   fInput.Assign(AContent);
   if fAutomatic then Execute;
end;

(****************************************************
 Procedure TInterpreter.create
 Unit      interpreter
 Project   CODER / Spirit GmbH
 Author    Christian Aust
 Date      28.01.99 17:04:27
 ****************************************************)
constructor TInterpreter.create(AOwner: TComponent);
begin
   inherited;

   fToken:= TStringlist.Create;
   fInput:= TInOutStrings.Create;
   fOutput:= TInOutStrings.Create;
   fDelimiter:= TInterpreterDelimiter.Create;
   fToken.Sorted:= true;
   fAutomatic:= true;
end;

(****************************************************
 Procedure TInterpreter.destroy
 Unit      interpreter
 Project   CODER / Spirit GmbH
 Author    Christian Aust
 Date      28.01.99 17:04:32
 ****************************************************)
destructor TInterpreter.destroy;
begin
   fDelimiter.Free;
   fOutput.Free;
   fInput.Free;
   fToken.Free;
   inherited destroy;
end;

end.
